#include "common.h"
#include "rtsp.h"

void rtsp_ErasePage2(uint16 addr);
uint32 rtsp_ReadLatch(uint16 addrhigh, uint16 addrlow);
void rtsp_WriteLatch(uint16 addrhigh, uint16 addrlow, uint16 hival, uint16 loval);
void rtsp_WriteMem(uint16 addrhigh, uint16 addrlow);

void rtsp_ErasePage(uint16 page) {
	if(page <= RTSP_PAGE_VAR) {
		rtsp_ErasePage2(page<<11);
	}
}

void rtsp_ReadPage(uint16 page, uint16 offset, uint16 end, byte* data) {	//data is a 3072-byte buffer
	for(; offset <= end; ++offset,data+=3) {
		uint32 l = rtsp_ReadLatch(0, (page<<11) | (offset<<1));
		data[0] = l;
		data[1] = l >> 8;
		data[2] = l >> 16;
	}
}

void rtsp_WritePage(uint16 page, byte* data, uint16 page_size) {
	if(page >= RTSP_PAGE_BOOTLOADER) return;
	if(page == 0) {
		//Preserve the GOTO instruction
		rtsp_ReadPage(0,0,1,data);
	}

	rtsp_ErasePage(page);
	
	int offset = 0;
	int limit = page_size/3;
	
	while(offset < limit) {
		uint16 w1hi = data[2];
		uint16 w1lo = (((uint16)(data[1]))<<8) | data[0];
		uint16 w2hi = data[5];
		uint16 w2lo = (((uint16)(data[4]))<<8) | data[3];

		uint16 addr = (page<<11)|(offset<<1);

		TBLPAG = 0xFA;
		__builtin_tblwtl(0,w1lo);
		__builtin_tblwth(0,w1hi);
		__builtin_tblwtl(2,w2lo);
		__builtin_tblwth(2,w2hi);
		rtsp_WriteMem(0, addr);
		TBLPAG = 0;

		data += 6; offset += 2;
	}
}

extern byte page_buffer[3072];
void rtsp_WriteConfigWord(uint16 value) {
	rtsp_ReadPage(RTSP_PAGE_VAR, 0, 1023, page_buffer);
	
	//Modify the contents
	page_buffer[3069] = (byte)value;
	page_buffer[3070] = value>>8;
	page_buffer[3071] = 0xFF;
	
	rtsp_WritePage(RTSP_PAGE_VAR, page_buffer, 3072);
}
